// Media_PI_KBDW
// USB Keyboard for Openelec PIAdam Tatuś

#include <IRremote.h>
#include <Keyboard.h>

//definicja przyporzadkowan klawiszy
#define SW_KEY_INFO    A0  // F7(36) "I"
#define SW_KEY_LEFT    13  // C7(32)
#define SW_KEY_RIGHT   A3  // F4(39)
#define SW_KEY_ESC     A1  // F6(37)
#define SW_KEY_UP      A2  // F5(38)
#define SW_KEY_DN      12  // D6(26)
#define SW_KEY_ENTER   6   // D7(27)
#define SW_KEY_PLAY    9   // B5(29) "P"/PLAY "Space"/PAUSE 
#define SW_KEY_FF      A4  // F1(40) "."
#define SW_KEY_REV     8   // B4(28) ","
#define SW_KEY_STOP    10  // B6(30) "X"

#define SW_KEY_SHDN    0  // B6(30) "ESC+s+Enter"

//definicja przyporzadkowan kodow RC5
#define IR_KEY_UP_ARROW         0x020
#define IR_KEY_DOWN_ARROW       0x021
#define IR_KEY_RETURN           0x022
#define IR_KEY_LEFT_ARROW       0x011
#define IR_KEY_RIGHT_ARROW      0x010
#define IR_KEY_ESC              0x039
#define IR_KEY_INFO             0x028
#define IR_KEY_FF               0x02A
#define IR_KEY_REV              0x035
#define IR_KEY_SKIP_REV         0x02B
#define IR_KEY_SKIP_FF          0x03C
#define IR_KEY_PLAY             0x029
#define IR_KEY_STOP             0x00F
#define IR_KEY_PAUSE            0x02E
#define IR_KEY_VOLUP            0x036
#define IR_KEY_VOLDN            0x037
#define IR_KEY_DELETE           0x03F
#define IR_KEY_HOME	            0x032
#define IR_KEY_END	            0x034

#define IR_KEY_SHDN             0x00C  // "ESC+s+Enter"

int KBDelay = 200;
int RECV_PIN = 11;
int PWOK_PIN = 1;
int LED_PIN = 5;
int RC5_FLAG = 0, cnt_UP = 0, cnt_DN = 0, cnt_PLAY = 0;
int cnt_LONG = 5;
int cnt_LONG_PLAY = 2;
int flag_Play = 0;

IRrecv irrecv(RECV_PIN);
decode_results results;

void setup()
{
  //    Serial.begin(9600);
  pinMode(PWOK_PIN, OUTPUT);
  digitalWrite(PWOK_PIN, HIGH);
  pinMode(LED_PIN, OUTPUT);
  digitalWrite(LED_PIN, HIGH);

  pinMode(SW_KEY_INFO, INPUT);
  digitalWrite(SW_KEY_INFO, HIGH);
  pinMode(SW_KEY_LEFT, INPUT);
  digitalWrite(SW_KEY_LEFT, HIGH);
  pinMode(SW_KEY_RIGHT, INPUT);
  digitalWrite(SW_KEY_RIGHT, HIGH);
  pinMode(SW_KEY_ESC, INPUT);
  digitalWrite(SW_KEY_ESC, HIGH);
  pinMode(SW_KEY_UP, INPUT);
  digitalWrite(SW_KEY_UP, HIGH);
  pinMode(SW_KEY_DN, INPUT);
  digitalWrite(SW_KEY_DN, HIGH);
  pinMode(SW_KEY_ENTER, INPUT);
  digitalWrite(SW_KEY_ENTER, HIGH);
  pinMode(SW_KEY_PLAY, INPUT);
  digitalWrite(SW_KEY_PLAY, HIGH);
  pinMode(SW_KEY_FF, INPUT);
  digitalWrite(SW_KEY_FF, HIGH);
  pinMode(SW_KEY_REV, INPUT);
  digitalWrite(SW_KEY_REV, HIGH);
  pinMode(SW_KEY_STOP, INPUT);
  digitalWrite(SW_KEY_STOP, HIGH);
  pinMode(SW_KEY_SHDN, INPUT);
  digitalWrite(SW_KEY_SHDN, HIGH);

  irrecv.enableIRIn(); // Start the receiver
}

void RC5_Check(decode_results *results) {
  if (results->decode_type == RC5)   {
    //        Serial.print("Decoded RC5: ");
    RC5_FLAG = 1;
  }
  else  {
    RC5_FLAG = 0;
  }
}

void kbd_read()
{
  if (digitalRead(SW_KEY_INFO) == 0)  // button pressed
  {
    Keyboard.write('i');
    delay(KBDelay);
  }
  if (digitalRead(SW_KEY_ESC) == 0)
  {
    Keyboard.press(KEY_ESC);
    delay(KBDelay);
    Keyboard.release(KEY_ESC);
  }
  if (digitalRead(SW_KEY_LEFT) == 0)
  {
    Keyboard.press(KEY_LEFT_ARROW);
    delay(KBDelay);
    Keyboard.release(KEY_LEFT_ARROW);
  }
  if (digitalRead(SW_KEY_UP) == 0)
  {
    if (cnt_UP > cnt_LONG) {
      Keyboard.press(KEY_PAGE_UP);
      delay(KBDelay);
      Keyboard.release(KEY_PAGE_UP);
    }
    else {
      Keyboard.press(KEY_UP_ARROW);
      delay(KBDelay);
      Keyboard.release(KEY_UP_ARROW);
    }
    cnt_UP++;
  }
  if (digitalRead(SW_KEY_UP) == 1) {
    cnt_UP = 0;
  } // reset nacisniecia długiego
  if (digitalRead(SW_KEY_DN) == 0)
  {
    if (cnt_DN > cnt_LONG) {
      Keyboard.press(KEY_PAGE_DOWN);
      delay(KBDelay);
      Keyboard.release(KEY_PAGE_DOWN);
    }
    else {
      Keyboard.press(KEY_DOWN_ARROW);
      delay(KBDelay);
      Keyboard.release(KEY_DOWN_ARROW);
    }
    cnt_DN++;
  }
  if (digitalRead(SW_KEY_DN) == 1) {
    cnt_DN = 0;
  } // reset nacisniecia długiego
  if (digitalRead(SW_KEY_RIGHT) == 0)
  {
    Keyboard.press(KEY_RIGHT_ARROW);
    delay(KBDelay);
    Keyboard.release(KEY_RIGHT_ARROW);
  }
  if (digitalRead(SW_KEY_ENTER) == 0)
  {
    Keyboard.press(KEY_RETURN);
    delay(KBDelay);
    Keyboard.release(KEY_RETURN);
  }
  if (digitalRead(SW_KEY_PLAY) == 0)
  {
    if (flag_Play == 0) {
      Keyboard.press('p'); //play
      delay(KBDelay);
      Keyboard.release('p');
      flag_Play = 1;
    }
    else {
      Keyboard.press(' '); //pause
      delay(KBDelay);
      Keyboard.release(' ');
      flag_Play = 0;
    }
  }
  if (digitalRead(SW_KEY_FF) == 0)
  {
    Keyboard.press('.');
    delay(KBDelay);
    Keyboard.release('.');
  }
  if (digitalRead(SW_KEY_REV) == 0)
  {
    Keyboard.press(',');
    delay(KBDelay);
    Keyboard.release(',');
  }
  if (digitalRead(SW_KEY_STOP) == 0)
  {
    Keyboard.press('x');
    delay(KBDelay);
    Keyboard.release('x');
    flag_Play = 0;
  }
  if (digitalRead(SW_KEY_SHDN) == 0)  //Shutdown
  {
    Keyboard.press(KEY_ESC);
    delay(KBDelay);
    Keyboard.release(KEY_ESC);
    Keyboard.press('s');
    delay(KBDelay);
    Keyboard.release('s');
    Keyboard.press(KEY_RETURN);
    delay(KBDelay);
    Keyboard.release(KEY_RETURN);
    digitalWrite(LED_PIN, LOW);
    delay(500);
    digitalWrite(LED_PIN, HIGH);
    delay(500);
    digitalWrite(LED_PIN, LOW);
    delay(500);
    digitalWrite(LED_PIN, HIGH);
    delay(500);
    digitalWrite(LED_PIN, LOW);
    delay(5000);
    digitalWrite(PWOK_PIN, LOW);
  }
}

void ir_receive()
{
  if (irrecv.decode(&results)) {
    results.value = results.value & 0x0FF;
    //        Serial.println(results.value, HEX);
    if (RC5_FLAG == 1) {
      switch (results.value) {
        case IR_KEY_UP_ARROW:
          {
            Keyboard.press(KEY_UP_ARROW);
            delay(KBDelay);
            Keyboard.release(KEY_UP_ARROW);
            break;
          }
        case IR_KEY_DOWN_ARROW:
          {
            Keyboard.press(KEY_DOWN_ARROW);
            delay(KBDelay);
            Keyboard.release(KEY_DOWN_ARROW);
            break;
          }
        case IR_KEY_RETURN:
          {
            Keyboard.press(KEY_RETURN);
            delay(KBDelay);
            Keyboard.release(KEY_RETURN);
            break;
          }
        case IR_KEY_LEFT_ARROW:
          {
            Keyboard.press(KEY_LEFT_ARROW);
            delay(KBDelay);
            Keyboard.release(KEY_LEFT_ARROW);
            break;
          }
        case IR_KEY_RIGHT_ARROW:
          {
            Keyboard.press(KEY_RIGHT_ARROW);
            delay(KBDelay);
            Keyboard.release(KEY_RIGHT_ARROW);
            break;
          }
        case IR_KEY_ESC:  //ESC
          {
            Keyboard.press(KEY_ESC);
            delay(KBDelay);
            Keyboard.release(KEY_ESC);
            break;
          }
        case IR_KEY_INFO:  //info
          {
            Keyboard.write('i');
            delay(KBDelay);
            break;
          }
        case IR_KEY_FF:  //ff >>
          {
            Keyboard.press('f');
            delay(KBDelay);
            Keyboard.release('f');
            break;
          }
        case IR_KEY_REV:  //rev <<
          {
            Keyboard.press('r');
            delay(KBDelay);
            Keyboard.release('r');
            break;
          }
        case IR_KEY_SKIP_REV:  //rev skip |<<
          {
            Keyboard.press(',');
            delay(KBDelay);
            Keyboard.release(',');
            break;
          }
        case IR_KEY_SKIP_FF:  //ff skip >>|
          {
            Keyboard.press('.');
            delay(KBDelay);
            Keyboard.release('.');
            break;
          }
        case IR_KEY_PLAY:  //play
          {
            Keyboard.press('p');
            delay(KBDelay);
            Keyboard.release('p');
            break;
          }
        case IR_KEY_STOP:  //stop
          {
            Keyboard.press('x');
            delay(KBDelay);
            Keyboard.release('x');
            break;
          }
        case IR_KEY_PAUSE:  //pause
          {
            Keyboard.press(' ');
            delay(KBDelay);
            Keyboard.release(' ');
            break;
          }
        case IR_KEY_VOLUP: //Vol+
          {
            Keyboard.press('+');
            delay(KBDelay);
            Keyboard.release('+');
            break;
          }
        case IR_KEY_VOLDN: //Vol-
          {
            Keyboard.press('-');
            delay(KBDelay);
            Keyboard.release('-');
            break;
          }
        case IR_KEY_DELETE: // Delete
          {
            Keyboard.press(KEY_DELETE);
            delay(KBDelay);
            Keyboard.release(KEY_DELETE);
            break;
          }
        case IR_KEY_HOME: // Home
          {
            Keyboard.press(KEY_HOME);
            delay(KBDelay);
            Keyboard.release(KEY_HOME);
            break;
          }
        case IR_KEY_END: // End
          {
            Keyboard.press(KEY_END);
            delay(KBDelay);
            Keyboard.release(KEY_END);
            break;
          }
        case IR_KEY_SHDN: // Shutdown
          {
            Keyboard.press(KEY_ESC);
            delay(KBDelay);
            Keyboard.release(KEY_ESC);
            Keyboard.press('s');
            delay(KBDelay);
            Keyboard.release('s');
            Keyboard.press(KEY_RETURN);
            delay(KBDelay);
            Keyboard.release(KEY_RETURN);
            digitalWrite(LED_PIN, LOW);
            delay(500);
            digitalWrite(LED_PIN, HIGH);
            delay(500);
            digitalWrite(LED_PIN, LOW);
            delay(500);
            digitalWrite(LED_PIN, HIGH);
            delay(500);
            digitalWrite(LED_PIN, LOW);
            delay(5000);
            digitalWrite(PWOK_PIN, LOW);
            break;
          }
      }
    }
    RC5_Check(&results);
    irrecv.resume();
  }
}

void loop() {
  ir_receive();
  delay(250);
  kbd_read();
}







